%============United Multi-Operator Evolutionary Algorithms==========

%==========================REFERENCE================================
%% S. Elsayed, R. Sarker, D. Essam and N. Hamza (2014).
% Testing United Multi-Operator Evolutionary Algorithms on The CEC2014 Real-Parameter Numerical Optimization.
%IEEE Congress on Evolutionary Computation, Beijing, China
%==========================================================================

%%==========================================================================
% Should you have any queries, please contact
% Dr. Saber Elsayed. University of New South Wales at Canberra
% s.elsayed@adfa.edu.au
% https://sites.google.com/site/saberelsayed3/home
%==========================================================================


function [x, fitx, success,prob1,prob2,count1,count2,bestold,bestx,col,mu,cr ] = ...
    SAMO_DE( x, fitx,success,prob1,~,bestold,bestx,col,mu,cr, ParamsFunc)
%fprintf('D');

global xmin xmax   n  PopSize ...
    f_optimal  current_eval I_fno max_eval

x_new=zeros(PopSize,n);
fitx_new=zeros(1,PopSize);
contribution=zeros(1,PopSize);
count1=0;count2=0;
F_i=zeros(1,PopSize);
Cr=zeros(1,PopSize);

%% generate random numbers
prob_all = rand(1,PopSize);
phi=zeros(1,PopSize);
for i=1:PopSize
    if prob_all(i)<=prob1
        percentage = 0.0+ rand * 0.5;
        phi(i) = ceil(percentage*PopSize) ;
    else
        phi(i) =1;
    end
end

for i=1:PopSize
    %% generate different random integer numbers
    randnum=zeros(1,2);
    for  t=1: 3
        similarity=1;
        while (similarity~=0)
            similarity=0;
            randnum(t) =ceil(PopSize.*rand);
            while ((randnum(t)==i) ||randnum(t)==phi(i)) || (any(randnum(1:t-1) == randnum(t)))
                randnum(t) =ceil(PopSize.*rand);
            end
        end
    end
    
    %% self-adaptively calculate F and Cr
    if rand<0.75
        F_i(i)= mu(randnum(3)) +  rand * ((mu(randnum(1)) - mu(randnum(2))));
        if F_i(i)<0.0
            F_i(i)= abs(F_i(i));
        elseif F_i(i)>1
            F_i(i)=2-F_i(i);
        end
        Cr(i) = cr(randnum(3)) +  rand * ((cr(randnum(1)) - cr(randnum(2))));
        
        if Cr(i)<0.0
            Cr(i)= abs(Cr(i));
        elseif Cr(i)>1
            Cr(i)=2-Cr(i);
        end
    else
        F_i(i)=rand;
        Cr(i)=rand;
    end
    %% apply DE1 -- DE/phi-rand/1
    if prob_all(i)<=prob1
        %% DE 1
        jrand =  ceil(n.*rand);
        for j=1:n
            if rand< Cr(i) || j== jrand
                x_new(i,j) = x(phi(i),j) +  F_i(i) * ((x(randnum(1),j) - x(randnum(2),j)));
            else
                x_new(i,j) = x(i,j);
            end
        end
        contribution(i)=1;
        
    else
        %% Apply DE2 -- DE/current-phi_best
        jrand =  ceil(n.*rand);
        for j=1:n
            if rand< Cr(i) || j== jrand
                x_new(i,j) = x(i,j) +  F_i(i) * ((x(randnum(1),j) - x(randnum(2),j))) +  F_i(i) * (x(phi(i),j) - x(i,j));
            else
                x_new(i,j) = x(i,j);
            end
        end
        contribution (i)=2;
    end
    [x_new, n, xmax, xmin] = han_boun (x_new, n, xmax, xmin, I_fno,i);
    %fitx_new(i) = cec14_func(x_new(i,:)',I_fno);
    if current_eval < ParamsFunc.FunctionEvaluations
    fitx_new(i) = feval(ParamsFunc.FevalName,ParamsFunc.FuncId, x_new(i,:));
    else
        x = []; fitx = 0; success = []; prob1 = []; prob2 = [];
        count1 = 0; count2 = []; bestold = 0; bestx = []; 
        col = []; mu = 0; cr = 0;
        return;
    end
    
    if fitx_new(i)> fitx(i)
        fitx_new (i) = fitx(i);
        x_new(i,:) = x(i,:);
        contribution(i)=0;
        F_i(i)=mu(i);
        Cr(i)=cr(i);
    else
        if  contribution(i)==1
            count1=count1+1;
        else
            count2=count2+1;
        end
    end
    %% update FITNESS EVALUATIONS
    current_eval =current_eval+1;
    
    %% check to print
    if (abs(fitx_new(i)-f_optimal)) < bestold
        bestold=abs((fitx_new(i)-f_optimal));
        bestx= x(i,:);
    end
    if mod(current_eval,0.01*max_eval)==0 && current_eval<=max_eval
        if abs(bestold)>1e-8
            %all_results(tt+1,col)= abs(bestold);
            col=col+1;
        else
            bestold=0;
            col=col+1;
            %all_results(tt+1,col:100)= 0;
            current_eval=max_eval;
        end
    end
end
%% update x and fitx
x = x_new;
fitx = fitx_new;
mu=F_i;
cr=Cr;
%% calc. prob1 and prob2
count1=count1/sum(prob_all<=prob1);
count2=count2/sum(prob_all>prob1);
prob1= max(0.05,min(0.95,count1/(count1+count2)));
prob2=1-prob1;

%% sort new pop, fitness, mu and cr
[fitx, ind]=sort(fitx);
x=x(ind,:);
mu=mu(ind);
cr=cr(ind);