%============United Multi-Operator Evolutionary Algorithms==========

%==========================REFERENCE================================
%% S. Elsayed, R. Sarker, D. Essam and N. Hamza (2014).
% Testing United Multi-Operator Evolutionary Algorithms on The CEC2014 Real-Parameter Numerical Optimization.
%IEEE Congress on Evolutionary Computation, Beijing, China
%==========================================================================

%%==========================================================================
% Should you have any queries, please contact
% Dr. Saber Elsayed. University of New South Wales at Canberra
% s.elsayed@adfa.edu.au
% https://sites.google.com/site/saberelsayed3/home
%==========================================================================

%% CMA-ES code is a modified version of that presented on (https://www.lri.fr/~hansen/cmaes_inmatlab.html#matlab).
%% This is to fit it to the problems on hand as well as the structure of the general algorthm's framework

function[ x, fitx, success, setting,bestold,bestx,col,count,bnd,fitness,sa] = ...
    SAMO_ES( ~, ~, setting,success, iter,bestold,bestx,col,count,fitness,bnd,sa, ParamsFunc)
%fprintf('E');
global xmin xmax   n  PopSize ...
    max_eval f_optimal  current_eval I_fno limit_all


stopOnWarnings=0;
noiseReevals = 0;

fitness.raw = NaN(1, PopSize + noiseReevals);
fitness.raw(PopSize + find(isnan(fitness.raw(1:noiseReevals)))) = NaN;
% flgEvalParallel=0;
arz=zeros(n,PopSize);
arx=zeros(n,PopSize);
arxvalid=zeros(n,PopSize);
for k=find(isnan(fitness.raw)),
    
    
    if k <= PopSize  % regular samples (not the re-evaluation-samples)
        arz(:,k) = randn(n,1); % (re)sample
        
        if setting.flgDiagonalOnly
            arx(:,k) = setting.xmean + setting.sigma * setting.diagD .* arz(:,k);
        else
            arx(:,k) = setting.xmean + setting.sigma * (setting.BD * arz(:,k));
        end
    else % re-evaluation solution with index > PopSize
        if setting.flgDiagonalOnly
            arx(:,k) = arx(:,k-PopSize) + (noiseEpsilon * setting.sigma) * setting.diagD .* randn(n,1);
        else
            arx(:,k) = arx(:,k-PopSize) + (noiseEpsilon * setting.sigma) * (setting.BD * randn(n,1));
        end
    end
    
    if ~bnd.isactive
        arxvalid(:,k) = arx(:,k);
    else
        arxvalid(:,k) = xintobounds(arx(:,k), xmin, xmax);
    end
    
    %% un comment for case 2
% %     if current_eval>limit_all
% %     %         arxvalid(:,k) = arxvalid(:,k)';
% %     else
% %          arxvalid(:,k) = arx(:,k)';
% %     end

%%
% Note that arxvalid=zeros(n,PopSize);
    %fitness.raw(k) = cec14_func(arxvalid(:,k),I_fno);%
    if current_eval < ParamsFunc.FunctionEvaluations
    fitness.raw(k) = feval(ParamsFunc.FevalName,ParamsFunc.FuncId, arxvalid(:,k)');
    else
        x = []; fitx = []; success = []; setting = []; bestold = [];
        bestx = []; col = []; count = []; bnd = []; fitness = 0; sa = [];
        return;
    end
        
    ans1 =arxvalid(:,k)>=xmin ;
    ans2 =arxvalid(:,k)<=xmax ;
    current_eval=current_eval+1; %% increase the fitness evaluations
    % check to print the periodic results
    if (abs(fitness.raw(k)-f_optimal)) < bestold  && min(ans1)>0 && min(ans2)>0
        
        bestold=abs((fitness.raw(k)-f_optimal));
        bestx=  arxvalid(:,k)';
    end
    
%     if mod(current_eval,0.01*max_eval)==0 && current_eval<=max_eval
%         if abs(bestold)>1e-8
%             all_results(tt+1,col)= abs(bestold);
%             col=col+1;
%         else
%             bestold=0;
%             col=col+1;
%             all_results(tt+1,col:100)= 0;
%             current_eval=max_eval;
%         end
%     end
end

fitness.sel = fitness.raw;
%%
% ----- handle boundaries -----
if 1 < 3 && bnd.isactive
    % Get delta fitness values
    val = myprctile(fitness.raw, [25 75]);
    % more precise would be exp(mean(log(setting.diagC)))
    val = (val(2) - val(1)) / n / mean(setting.diagC) / setting.sigma^2;
    %val = (myprctile(fitness.raw, 75) - myprctile(fitness.raw, 25)) ...
    %    / n / mean(setting.diagC) / setting.sigma^2;
    % Catch non-sensible values
    if ~isfinite(val)
        %         warning('Non-finite fitness range');
        val = max(bnd.dfithist);
    elseif val == 0 % happens if all points are out of bounds
        val = min(bnd.dfithist(bnd.dfithist>0));  % seems not to make sense, given all solutions are out of bounds
    elseif bnd.validfitval == 0 % flag that first sensible val was found
        bnd.dfithist = [];
        bnd.validfitval = 1;
    end
    
    % Store delta fitness values
    if length(bnd.dfithist) < 20+(3*n)/PopSize
        bnd.dfithist = [bnd.dfithist val];
    else
        bnd.dfithist = [bnd.dfithist(2:end) val];
    end
    
    [tx, ti]  = xintobounds(setting.xmean, xmin, xmax);
    
    % Set initial setting.weights
    if bnd.iniphase
        if any(ti)
            bnd.weights(bnd.isbounded) = 2.0002 * median(bnd.dfithist);
            if bnd.flgscale == 0 % scale only initial setting.weights then
                dd = setting.diagC;
                idx = find(bnd.isbounded);
                dd = dd(idx) / mean(dd); %  remove mean scaling
                bnd.weights(idx) = bnd.weights(idx) ./ dd;
            end
            if bnd.validfitval && iter > 2
                bnd.iniphase = 0;
            end
        end
    end
    
    % Increase setting.weights
    if  1 < 3 && any(ti) % any coordinate of setting.xmean out of bounds
        % judge distance of setting.xmean to boundary
        tx = setting.xmean - tx;
        idx = (ti ~= 0 & abs(tx) > 3*max(1,sqrt(n)/setting.mueff) ...
            * setting.sigma*sqrt(setting.diagC)) ;
        % only increase if setting.xmean is moving away
        idx = idx & (sign(tx) == sign(setting.xmean - setting.xold));
        if ~isempty(idx) % increase
            % the factor became 1.2 instead of 1.1, because
            % changed from max to min in version 3.52
            bnd.weights(idx) = 1.2^(min(1, setting.mueff/10/n)) * bnd.weights(idx);
        end
    end
    
    % Calculate scaling biased to unity, product is one
    if bnd.flgscale ~= 0
        bnd.scale = exp(0.9*(log(setting.diagC)-mean(log(setting.diagC))));
    end
    
    % Assigned penalized fitness
    bnd.arpenalty = (bnd.weights ./ bnd.scale)' * (arxvalid - arx).^2;
    
    fitness.sel = fitness.raw + bnd.arpenalty;
    
end % handle boundaries
% ----- end handle boundaries -----


% Sort by fitness
[fitness.raw, fitness.idx] = sort(fitness.raw);
[fitness.sel, fitness.idxsel] = sort(fitness.sel);  % minimization
fitness.hist(2:end) = fitness.hist(1:end-1);    % record short history of
fitness.hist(1) = fitness.raw(1);               % best fitness values
if length(fitness.histbest) < 120+ceil(30*n/PopSize) || ...
        (mod(iter, 5) == 0  && length(fitness.histbest) < 2e4)  % 20 percent of 1e5 gen.
    fitness.histbest = [fitness.raw(1) fitness.histbest];          % best fitness values
    fitness.histmedian = [median(fitness.raw) fitness.histmedian]; % median fitness values
else
    fitness.histbest(2:end) = fitness.histbest(1:end-1);
    fitness.histmedian(2:end) = fitness.histmedian(1:end-1);
    fitness.histbest(1) = fitness.raw(1);           % best fitness values
    fitness.histmedian(1) = median(fitness.raw);    % median fitness values
end
fitness.histsel(2:end) = fitness.histsel(1:end-1); % record short history of
fitness.histsel(1) = fitness.sel(1);               % best sel fitness values

% Calculate new setting.xmean, this is selection and recombination
setting.xold = setting.xmean; % for speed up of Eq. (2) and (3)
cmean =1;% 1/min(max((PopSize-1*n)/2, 1), n);  % == 1/kappa
setting.xmean = (1-cmean) * setting.xold + cmean * arx(:,fitness.idxsel(1:setting.mu))*setting.weights;
zmean = arz(:,fitness.idxsel(1:setting.mu))*setting.weights;%==D^-1*setting.B'*(setting.xmean-setting.xold)/setting.sigma


% Cumulation: update evolution paths
setting.ps = (1-setting.cs)*setting.ps + sqrt(setting.cs*(2-setting.cs)*setting.mueff) * (setting.B*zmean);          % Eq. (4)
hsig = norm(setting.ps)/sqrt(1-(1-setting.cs)^(2*iter))/setting.chiN < 1.4 + 2/(n+1);

setting.pc = (1-setting.cc)*setting.pc ...
    + hsig*(sqrt(setting.cc*(2-setting.cc)*setting.mueff)/setting.sigma/cmean) * (setting.xmean-setting.xold);     % Eq. (2)
if hsig == 0
    % disp([num2str(iter) ' ' num2str(counteval) ' setting.pc update stalled']);
end

% Adapt covariance matrix
neg.ccov = 0;  % TODO: move parameter setting upwards at some point
if setting.ccov1 + setting.ccovmu > 0                                                    % Eq. (3)
    if setting.flgDiagonalOnly % internal linear(?) complexity
        setting.diagC = (1-setting.ccov1_sep-setting.ccovmu_sep+(1-hsig)*setting.ccov1_sep*setting.cc*(2-setting.cc)) * setting.diagC ... % regard old matrix
            + setting.ccov1_sep * setting.pc.^2 ...               % plus rank one update
            + setting.ccovmu_sep ...                      % plus rank setting.mu update
            * (setting.diagC .* (arz(:,fitness.idxsel(1:setting.mu)).^2 * setting.weights));
        %             * (repmat(setting.diagC,1,setting.mu) .* arz(:,fitness.idxsel(1:setting.mu)).^2 * setting.weights);
        setting.diagD = sqrt(setting.diagC); % replaces eig(setting.C)
    else
        arpos = (arx(:,fitness.idxsel(1:setting.mu))-repmat(setting.xold,1,setting.mu)) / setting.sigma;
        setting.C = (1-setting.ccov1-setting.ccovmu+(1-hsig)*setting.ccov1*setting.cc*(2-setting.cc)) * setting.C ... % regard old matrix
            + setting.ccov1 * setting.pc*setting.pc' ...     % plus rank one update
            + setting.ccovmu ...             % plus rank setting.mu update
            * arpos * (repmat(setting.weights,1,n) .* arpos');
        % is now O(setting.mu*n^2 + setting.mu*n), was O(setting.mu*n^2 + setting.mu^2*n) when using diag(setting.weights)
        %   for setting.mu=30*n it is now 10 times faster, overall 3 times faster
        
        setting.diagC = diag(setting.C);
    end
end

% the following is de-preciated and will be removed in future
% better setting for setting.cc makes this hack obsolete
if 11 < 2 && ~flgscience
    % remove momentum in setting.ps, if setting.ps is large and fitness is getting worse.
    % this should rarely happen.
    % this might very well be counterproductive in dynamic environments
    if sum(setting.ps.^2)/n > 1.5 + 10*(2/n)^.5 && ...
            fitness.histsel(1) > max(fitness.histsel(2:3))
        setting.ps = setting.ps * sqrt(n*(1+max(0,log(sum(setting.ps.^2)/n))) / sum(setting.ps.^2));
        if flgdisplay
            %             disp(['Momentum in setting.ps removed at [niter neval]=' ...
            %                 num2str([iter counteval]) ']']);
        end
    end
end

% Adapt setting.sigma
setting.sigma = setting.sigma * exp(min(1, (sqrt(sum(setting.ps.^2))/setting.chiN - 1) * setting.cs/setting.damps));             % Eq. (5)
% disp([iter norm(setting.ps)/setting.chiN]);

if 11 < 3   % testing with optimal step-size
    if iter == 1
        disp('*********** setting.sigma set to const * ||x|| ******************');
    end
    setting.sigma = 0.04 * setting.mueff * sqrt(sum(setting.xmean.^2)) / n; % 20D,lam=1000:25e3
    setting.sigma = 0.3 * setting.mueff * sqrt(sum(setting.xmean.^2)) / n; % 20D,lam=(40,1000):17e3
    %      75e3 with def (1.5)
    %      35e3 with setting.damps=0.25
end
if 11 < 3
    if iter == 1
        disp('*********** setting.xmean set to const ******************');
    end
    setting.xmean = ones(n,1);
end

% Update setting.B and D from setting.C

if ~setting.flgDiagonalOnly && (setting.ccov1+setting.ccovmu+neg.ccov) > 0 && mod(iter, 1/(setting.ccov1+setting.ccovmu+neg.ccov)/n/10) < 1
    setting.C=triu(setting.C)+triu(setting.C,1)'; % enforce symmetry to prevent complex numbers
    [setting.B,tmp] = eig(setting.C);     % eigen decomposition, setting.B==normalized eigenvectors
    % effort: approx. 15*n matrix-vector multiplications
    setting.diagD = diag(tmp);
    
    if any(~isfinite(setting.diagD))
        clear idx; % prevents error under octave
        save(['tmp' opts.SaveFilename]);
        error(['function eig returned non-finited eigenvalues, cond(setting.C)=' ...
            num2str(cond(setting.C)) ]);
    end
    if any(any(~isfinite(setting.B)))
        clear idx; % prevents error under octave
        save(['tmp' opts.SaveFilename]);
        error(['function eig returned non-finited eigenvectors, cond(setting.C)=' ...
            num2str(cond(setting.C)) ]);
    end
    
    % limit condition of setting.C to 1e14 + 1
    if min(setting.diagD) <= 0
        if stopOnWarnings
            %             stopflag(end+1) = {'warnconditioncov'};
        else
            %             warning(['Iteration ' num2str(iter) ...
            %                 ': Eigenvalue (smaller) zero']);
            setting.diagD(setting.diagD<0) = 0;
            tmp = max(setting.diagD)/1e14;
            setting.C = setting.C + tmp*eye(n,n); setting.diagD = setting.diagD + tmp*ones(n,1);
        end
    end
    if max(setting.diagD) > 1e14*min(setting.diagD)
        if stopOnWarnings
            %             stopflag(end+1) = {'warnconditioncov'};
        else
            %             warning(['Iteration ' num2str(iter) ': condition of setting.C ' ...
            %                 'at upper limit' ]);
            tmp = max(setting.diagD)/1e14 - min(setting.diagD);
            setting.C = setting.C + tmp*eye(n,n); setting.diagD = setting.diagD + tmp*ones(n,1);
        end
    end
    
    setting.diagC = diag(setting.C);
    setting.diagD = sqrt(setting.diagD); % D contains standard deviations now
    % setting.diagD = setting.diagD / prod(setting.diagD)^(1/n);  setting.C = setting.C / prod(setting.diagD)^(2/n);
    setting.BD = setting.B.*repmat(setting.diagD',n,1); % O(n^2)
end % if mod

% Align/rescale order of magnitude of scales of setting.sigma and setting.C for nicer output
% TODO: interference with sigmafacup: replace 1e10 with 2*sigmafacup
% not a very usual case
if 1 < 2 && setting.sigma > 1e10*max(setting.diagD) && setting.sigma > 8e14 * max(setting.insigma)
    fac = setting.sigma; % / max(setting.diagD);
    setting.sigma = setting.sigma/fac;
    setting.pc = fac * setting.pc;
    setting.diagD = fac * setting.diagD;
    if ~setting.flgDiagonalOnly
        setting.C = fac^2 * setting.C; % disp(fac);
        setting.BD = setting.B .* repmat(setting.diagD',n,1); % O(n^2), but repmat might be inefficient todo?
    end
    setting.diagC = fac^2 * setting.diagC;
end

if setting.flgDiagonalOnly > 1 && iter > setting.flgDiagonalOnly
    % full covariance matrix from now on
    setting.flgDiagonalOnly = 0;
    setting.B = eye(n,n);
    setting.BD = diag(setting.diagD);
    setting.C = diag(setting.diagC); % is better, because correlations are spurious anyway
end

% ----- numerical error management -----
% Adjust maximal coordinate axis deviations
if any(setting.sigma*sqrt(setting.diagC) > setting.maxdx)
    setting.sigma = min(setting.maxdx ./ sqrt(setting.diagC));
    %warning(['Iteration ' num2str(iter) ': coordinate axis std ' ...
    %         'deviation at upper limit of ' num2str(setting.maxdx)]);
    % stopflag(end+1) = {'maxcoorddev'};
end
% Adjust minimal coordinate axis deviations
if any(setting.sigma*sqrt(setting.diagC) < setting.mindx)
    setting.sigma = max(setting.mindx ./ sqrt(setting.diagC)) * exp(0.05+setting.cs/setting.damps);
    %warning(['Iteration ' num2str(iter) ': coordinate axis std ' ...
    %         'deviation at lower limit of ' num2str(setting.mindx)]);
    % stopflag(end+1) = {'mincoorddev'};;
end
% Adjust too low coordinate axis deviations
if any(setting.xmean == setting.xmean + 0.2*setting.sigma*sqrt(setting.diagC))
    if stopOnWarnings
        %         stopflag(end+1) = {'warnnoeffectcoord'};
    else
        %         warning(['Iteration ' num2str(iter) ': coordinate axis std ' ...
        %             'deviation too low' ]);
        if setting.flgDiagonalOnly
            setting.diagC = setting.diagC + (setting.ccov1_sep+setting.ccovmu_sep) * (setting.diagC .* ...
                (setting.xmean == setting.xmean + 0.2*setting.sigma*sqrt(setting.diagC)));
        else
            setting.C = setting.C + (setting.ccov1+setting.ccovmu) * diag(setting.diagC .* ...
                (setting.xmean == setting.xmean + 0.2*setting.sigma*sqrt(setting.diagC)));
        end
        setting.sigma = setting.sigma * exp(0.05+setting.cs/setting.damps);
    end
end
% Adjust step size in case of (numerical) precision problem
if setting.flgDiagonalOnly
    tmp = 0.1*setting.sigma*setting.diagD;
else
    tmp = 0.1*setting.sigma*setting.BD(:,1+floor(mod(iter,n)));
end
if all(setting.xmean == setting.xmean + tmp)
    %     ii = 1+floor(mod(iter,n));
    if stopOnWarnings
    else
        setting.sigma = setting.sigma * exp(0.2+setting.cs/setting.damps);
    end
end
% Adjust step size in case of equal function values (flat fitness)
% isequalfuncvalues = 0;
if fitness.sel(1) == fitness.sel(1+ceil(0.1+PopSize/4))
    % isequalfuncvalues = 1;
    if setting.stopOnEqualFunctionValues
        setting.arrEqualFunvals = [iter setting.arrEqualFunvals(1:end-1)];
        % stop if this happens in more than 33%
        if setting.arrEqualFunvals(end) > iter - 3 * length(setting.arrEqualFunvals)
            %             stopflag(end+1) = {'equalfunvals'};
        end
    else
        if flgWarnOnEqualFunctionValues
            %             warning(['Iteration ' num2str(iter) ...
            %                 ': equal function values f=' num2str(fitness.sel(1)) ...
            %                 ' at maximal main axis setting.sigma ' ...
            %                 num2str(setting.sigma*max(setting.diagD))]);
        end
        setting.sigma = setting.sigma * exp(0.2+setting.cs/setting.damps);
    end
end
% Adjust step size in case of equal function values
if iter > 2 && myrange([fitness.hist fitness.sel(1)]) == 0
    if stopOnWarnings
        % 	stopflag(end+1) = {'warnequalfunvalhist'};
    else
        %         warning(['Iteration ' num2str(iter) ...
        %             ': equal function values in history at maximal main ' ...
        %             'axis setting.sigma ' num2str(setting.sigma*max(setting.diagD))]);
        setting.sigma = setting.sigma * exp(0.2+setting.cs/setting.damps);
    end
end

%% print out final results
x= arxvalid';
% fold=fitx(1);
fitx= fitness.raw;

if fitx(1)>bestold+f_optimal
    sa=sa+1;
else
    sa=0;
end


function [x, idx] = xintobounds(x, xmin, xmax)
%
% x can be a column vector or a matrix consisting of column vectors
%
if ~isempty(xmin)
    if length(xmin) == 1
        idx = x < xmin;
        x(idx) = min(xmax,max(xmin, 2*xmin-x(idx)));
    else
        arbounds = repmat(xmin, 1, size(x,2));
        idx = x < arbounds;
        x(idx) = arbounds(idx);
    end
else
    idx = 0;
end
if ~isempty(xmax)
    if length(xmax) == 1
        idx2 = x > xmax;
        x(idx2) = max(xmin,min(xmax, 2*xmax-x(idx2)));
    else
        arbounds = repmat(xmax, 1, size(x,2));
        idx2 = x > arbounds;
        x(idx2) = arbounds(idx2);
    end
else
    idx2 = 0;
end
idx = idx2-idx;

% ---------------------------------------------------------------
% ----- replacements for statistic toolbox functions ------------
% ---------------------------------------------------------------
% ---------------------------------------------------------------
function res=myrange(x)
res = max(x) - min(x);

% ---------------------------------------------------------------
% ---------------------------------------------------------------
function res = myprctile(inar, perc, idx)
%
% Computes the percentiles in vector perc from vector inar
% returns vector with length(res)==length(perc)
% idx: optional index-array indicating sorted order
%

N = length(inar);
flgtranspose = 0;

% sizes
if size(perc,1) > 1
    perc = perc';
    flgtranspose = 1;
    if size(perc,1) > 1
        error('perc must not be a matrix');
    end
end
if size(inar, 1) > 1 && size(inar,2) > 1
    error('data inar must not be a matrix');
end

% sort inar
if nargin < 3 || isempty(idx)
    [sar] = sort(inar);
else
    sar = inar(idx);
end

res=[];
for p = perc
    if p <= 100*(0.5/N)
        res(end+1) = sar(1);
    elseif p >= 100*((N-0.5)/N)
        res(end+1) = sar(N);
    else
        % find largest index smaller than required percentile
        availablepercentiles = 100*((1:N)-0.5)/N;
        i = find(p > availablepercentiles, 1, 'last' );
        % interpolate linearly
        res(end+1) = sar(i) ...
            + (sar(i+1)-sar(i))*(p - availablepercentiles(i)) ...
            / (availablepercentiles(i+1) - availablepercentiles(i));
        
    end
end

if flgtranspose
    res = res';
end


% ---------------------------------------------------------------
% CMA-ES: Evolution Strategy with Covariance Matrix Adaptation for
% nonlinear function minimization. To be used under the terms of the
% GNU General Public License (http://www.gnu.org/copyleft/gpl.html).
% Author (copyright): Nikolaus Hansen, 2001-2008.
% e-mail: nikolaus.hansen AT inria.fr
% URL:http://www.bionik.tu-berlin.de/user/niko
% References: See below.
% ---------------------------------------------------------------
%
% GENERAL PURPOSE: The CMA-ES (Evolution Strategy with Covariance
% Matrix Adaptation) is a robust search method which should be
% applied, if derivative based methods, e.g. quasi-Newton BFGS or
% conjucate gradient, (supposably) fail due to a rugged search
% landscape (e.g. noise, local optima, outlier, etc.). On smooth
% landscapes CMA-ES is roughly ten times slower than BFGS. For up to
% N=10 variables even the simplex direct search method (Nelder & Mead)
% is often faster, but far less robust than CMA-ES.  To see the
% advantage of the CMA, it will usually take at least 30*N and up to
% 300*N function evaluations, where N is the search problem dimension.
% On considerably hard problems the complete search (a single run) is
% expected to take at least 30*N^2 and up to 300*N^2 function
% evaluations.
%
% SOME MORE COMMENTS:
% The adaptation of the covariance matrix (e.g. by the CMA) is
% equivalent to a general linear transformation of the problem
% coding. Nevertheless every problem specific knowlegde about the best
% linear transformation should be exploited before starting the
% search. That is, an appropriate a priori transformation should be
% applied to the problem. This also makes the identity matrix as
% initial covariance matrix the best choice.
%
% The strategy parameter lambda (population size, opts.PopSize) is the
% preferred strategy parameter to play with.  If results with the
% default strategy are not satisfactory, increase the population
% size. (Remark that the crucial parameter setting.mu (opts.ParentNumber) is
% increased proportionally to lambda). This will improve the
% strategies capability of handling noise and local minima. We
% recomment successively increasing lambda by a factor of about three,
% starting with initial values between 5 and 20. Casually, population
% sizes even beyond 1000+100*N can be sensible.
%
%
% ---------------------------------------------------------------
%%% REFERENCES
%
% The equation numbers refer to
% Hansen, N. and S. Kern (2004). Evaluating the CMA Evolution
% Strategy on Multimodal Test Functions.  Eighth International
% Conference on Parallel Problem Solving from Nature PPSN VIII,
% Proceedings, pp. 282-291, Berlin: Springer.
% (http://www.bionik.tu-berlin.de/user/niko/ppsn2004hansenkern.pdf)
%
% Further references:
% Hansen, N. and A. Ostermeier (2001). Completely Derandomized
% Self-Adaptation in Evolution Strategies. Evolutionary Computation,
% 9(2), pp. 159-195.
% (http://www.bionik.tu-berlin.de/user/niko/cmaartic.pdf).
%
% Hansen, N., S.D. Mueller and P. Koumoutsakos (2003). Reducing the
% Time Complexity of the Derandomized Evolution Strategy with
% Covariance Matrix Adaptation (CMA-ES). Evolutionary Computation,
% 11(1).  (http://mitpress.mit.edu/journals/pdf/evco_11_1_1_0.pdf).
%
% Ros, R. and N. Hansen (2008). A Simple Modification in CMA-ES
% Achieving Linear Time and Space Complexity. To appear in Tenth
% International Conference on Parallel Problem Solving from Nature
% PPSN X, Proceedings, Berlin: Springer.
%
% Hansen, N., A.S.P. Niederberger, L. Guzzella and P. Koumoutsakos
% (2009?). A Method for Handling Uncertainty in Evolutionary
% Optimization with an Application to Feedback Control of
% Combustion. To appear in IEEE Transactions on Evolutionary
% Computation.



