%============United Multi-Operator Evolutionary Algorithms==========

%==========================REFERENCE================================
%% S. Elsayed, R. Sarker, D. Essam and N. Hamza (2014).
% Testing United Multi-Operator Evolutionary Algorithms on The CEC2014 Real-Parameter Numerical Optimization.
%IEEE Congress on Evolutionary Computation, Beijing, China
%==========================================================================

%%==========================================================================
% Should you have any queries, please contact
% Dr. Saber Elsayed. University of New South Wales at Canberra
% s.elsayed@adfa.edu.au
% https://sites.google.com/site/saberelsayed3/home
%==========================================================================


function [x, fitx, success, prob1,prob2,bestold,bestx,col] = ...
    SAMO_GA( x, fitx,success, iter, T,prob1,~,bestold,bestx,col, ParamsFunc)
%fprintf('G');
global xmin xmax   n  PopSize ...
    max_eval  f_optimal current_eval I_fno

%% define variables
arc_size= ceil(PopSize/2);
archive=zeros(arc_size,n);
x_new=zeros(PopSize,n);
archive(1:arc_size,:) = x(1:arc_size,:);
fitx_new=zeros(1,PopSize);

contribution=zeros(PopSize,1);
arch_size = size(archive);
% count1=0;count2=0;
i=1;

%% main loop
while i<=PopSize
    
    %% apply GA1
    if rand <prob1
        beta = normrnd(0.7,0.1);  %% generate beta = Gaussian number, with mean=0.7 and standard deviation=0.1.%%%%
        TcSize =randi([2,3]);     %% select the TC size 2 or 3, randomly.
        best=1000*ones(3,1);
        for j=1:3
            r=randperm(PopSize);
            randnum=r(1:TcSize);
            best(j) = min(randnum); %% as we sorted the array before, the better is the one with minimum randnum
        end
        
        a = best(1:3);
        %% sort the selected three parents
        a= sort(a);
        
        %% Check if there are similar
        if (a(1)== a(2))
            while (a(2)== a(1)) ||(a(2)== a(3))
                a(2)=randi(PopSize);
            end
            a= sort(a);
        end
        if (a(1)== a(3))
            while (a(3)== a(1)) ||(a(3)== a(2))
                a(3)= randi(PopSize);%
            end
            a= sort(a);
        end
        if (a(2)== a(3))
            while (a(3)== a(1)) ||(a(3)== a(2))
                a(3)= randi(PopSize);
            end
            a= sort(a);
        end
        %% Cr=100 for MPC-GA
        x_new(i,:) = x(a(1), :) + beta * (x(a(2), :) - x(a(3), :));
        x_new(i+1,:) = x(a(2), :) + beta * (x(a(3), :) - x(a(1), :));
        x_new(i+2,:) = x(a(3), :) + beta * (x(a(1), :) - x(a(2), :));
        for i_p=i:i+2
            [x_new, n, xmax, xmin] = han_boun (x_new, n, xmax, xmin, I_fno,i_p);
        end
        %% Randomized Operator
        for i_p=i:min(PopSize,i+2)
            p=0.1;
            for j=1: n
                if rand<p
                    x_new(i_p,j) =archive(ceil(arch_size(1).*rand),j);
                end
            end
            
            %% fitness evaluations
            %fitx_new(i_p) = cec14_func(x_new(i_p,:)',I_fno);
            if current_eval < ParamsFunc.FunctionEvaluations
            fitx_new(i_p) = feval(ParamsFunc.FevalName,ParamsFunc.FuncId, x_new(i_p,:));
            else
                x = []; fitx = 0; success = [];
                prob1 = 0; prob2 = 0; bestold = 0; bestx = []; col = 0;
                return;
            end
            current_eval= current_eval+1;
            
            %% check to print
            if (abs(fitx(i_p)-f_optimal)) < bestold
                bestold=abs((fitx(i_p)-f_optimal));
                bestx= x(i_p,:);
            end
%             if mod(current_eval,0.01*max_eval)==0&& current_eval<=max_eval
%                 if abs(bestold)>1e-8
%                     all_results(tt+1,col)= abs(bestold);
%                     col=col+1;
%                 else
%                     bestold=0;
%                     col=col+1;
%                     all_results(tt+1,col:100)= 0;
%                     current_eval=max_eval;
%                 end
%             end
            contribution (i_p)=1;
        end
        i=i+3;
    else
        %% Apply GA2 - SBX+NUM
        TcSize =randi([2,3]); %% select the TC size 2 or 3, randomly.
        best=zeros(2,1);
        for j=1:2
            r=randperm(PopSize); 
            randnum=r(1:TcSize);
            best(j) = min(randnum); %%% as we sorted the array before, the better is the one with minimum randnum
        end
        
        a = best(1:2);
        if (a(1)== a(2))
            while a(2)== a(1)
                a(2)=randi(PopSize);
            end
        end
        
        %% SBX crossover
        mu = 3;
        u = rand;
        %% calc beta
        if (u <= 0.5)
            beta2 = (2 * u)^ (1 / (1 + mu));
        else
            beta2 = (1 /(2*(1 - u)))^ (1 / (1 + mu));
        end
        x_new(i,:)= 0.5 * (((1 + beta2) * x(a(1), :)) + ((1 - beta2) * x(a(2), :)));
        x_new(i+1,:)= 0.5 * (((1 - beta2) * x(a(1), :)) + ((1 + beta2) * x(a(2), :)));
        
        
        %% non-uniform mutation
        for i_p=i:min(PopSize,i+1)
            %% check boundary
            [x_new, n, xmax, xmin] = han_boun (x_new, n, xmax, xmin, I_fno,i_p);
            
            for j=1: n
                if (rand<0.1)
                    lambda = (1 - (iter/T))^ 5; %% b=5
                    if (rand <= 0.5)
                        delta = (xmax - x_new(i_p, j)) * (1 - rand^ lambda);
                    else
                        delta = (xmin - x_new(i_p, j)) *(1 - (rand ^lambda));
                    end
                    x_new (i_p,j) = x_new(i_p,j) + delta;
                end
            end
            %% check boundary
            [x_new, n, xmax, xmin] = han_boun (x_new, n, xmax, xmin, I_fno,i_p);
            
            %% Calculate the objective function for every new offspring and update fitness evaluations
            % fitx_new(i_p) = cec14_func(x_new(i_p,:)',I_fno);
            if current_eval < ParamsFunc.FunctionEvaluations
            fitx_new(i_p) = feval(ParamsFunc.FevalName,ParamsFunc.FuncId, x_new(i_p,:));
            else                
                x = []; fitx = 0; success = [];
                prob1 = 0; prob2 = 0; bestold = 0; bestx = []; col = 0;
                return;
            end
            current_eval= current_eval+1;
            %% check to print
            if (abs(fitx_new(i_p)-f_optimal)) < bestold
                bestold=abs((fitx_new(i_p)-f_optimal));
                bestx= x(i_p,:);
            end
%             if mod(current_eval,0.01*max_eval)==0&& current_eval<=max_eval
%                 if abs(bestold)>1e-8
%                     all_results(tt+1,col)= abs(bestold);
%                     col=col+1;
%                 else
%                     bestold=0;
%                     col=col+1;
%                     all_results(tt+1,col:100)= 0;
%                     current_eval=max_eval;
%                 end
%             end
            contribution (i_p)=2;
        end
        i=i+2;
    end
end


count1=sum(contribution==1);

FSIZE=ceil(count1/2);
fx (1:FSIZE,:) = x (1:FSIZE,:);
ff (1:FSIZE) = fitx (1:FSIZE);
fc=zeros(1,FSIZE);

xx=[fx ;x_new];
fff=[ff fitx_new];
fcc= [contribution' fc];


[fff, ind]=sort(fff);
xx=xx(ind,:);
fcc=fcc(ind);

x (1:PopSize,:) = xx (1:PopSize,:)  ;
fitx(1:PopSize)  = fff(1:PopSize);
contribution(1:PopSize) = fcc(1:PopSize);
count1=sum(contribution==1);
count2=sum(contribution==2);



prob1= max(0.05,min(0.95,count1/ (count1+count2)));
prob2=1-prob1;

if count1==0 && count2==0
    prob1=0.5; prob2=0.5;
end

