
%====================United Multi-Operator Evolutionary Algorithms=================
%============Black-Box Optimization Benchmarking at CEC'2015 (CEC-BBOB)============

%==========================REFERENCE================================
%% S. Elsayed, R. Sarker, D. Essam and N. Hamza (2014).
% Testing United Multi-Operator Evolutionary Algorithms on The CEC2014 Real-Parameter Numerical Optimization.
%IEEE Congress on Evolutionary Computation, Beijing, China
%==========================================================================

%%==========================================================================
% Should you have any queries, please contact
% Dr. Saber Elsayed. University of New South Wales at Canberra
% s.elsayed@adfa.edu.au
% https://sites.google.com/site/saberelsayed3/home
%==========================================================================

%%================================================================================================
% For problems from   0 to 499, CS = 0; indx = 2; Popsize = 4 + floor(3 * log(dimension));
% For problems from 500 to 899, CS = 0; indx = 2; Popsize = 100;
% For problems from 900 to 999, CS = 50; indx = 0; Popsize = 100;
%=================================================================================================

%%================================================================================================
% For questions of parameters' settings in CEC-BBOB 2015, please contact
% Dr. Chao Yu. Peking University at Beijing
% chaoyu@pku.edu.cn
% (Website of fireworks algorithm) http://www.cil.pku.edu.cn/research/fwa/index.html
%=================================================================================================
% clear all

function UMOEAs_Main(ParamsFunc)

%-----------------------------------------------------------------------------------------------------
%% open the file with write permission
global xmin xmax   n  PopSize ...
    max_eval  n_opr  n1 n2 n3 ...
    f_optimal CS current_eval I_fno
%xmin=-100;    xmax=100;
xmin = 0;  xmax = 1;
n = ParamsFunc.Dim;                           %% dimension! change to 30, 50, 100
PopSize= 100;                    %% population size
max_eval = ParamsFunc.FunctionEvaluations; %10000*n;             %% max fitness evaluations
T= floor(max_eval/PopSize);            %% Max generations number for GA-SBX-NUM
limit_all=floor(max_eval/2);           %% upper limit to use multiple MOEAs. After it, the best MOEA runs alone
%optimality= 100:100:3000;       %% define the optimal solution as shown in the TR
n_opr=3;                        %% number of MOEAs and each MOEA's size
n1=ceil(PopSize);    n2=ceil(PopSize);    n3=PopSize;
indx= 0; %0;                         %% 1 = SAMODE, 2 = SAMOES and 3=SAMOGA
I_fno=0;                        %% problem number
run=0;
f_optimal = 0;

    %% some defenition at each run
    % ----------------------------------------------------------
    col=1;              %% to print in the first column in all_results.mat
    run=run+1;          %% increase runs by 1
    iter=0;             %% current generation
    %fprintf('Problem: %d \t run: %d  --> open periodic.txt to see results or wait until UMOEA finishes and open results_stat_.txt \n',I_fno,run)
    rand('state', sum(clock)); %% Define a random seed
    iter = iter+1;       %% Increase iter by 1
    success=zeros(1,3);  %% To save the success of each MOEA
    count_iter=0;        %% to be used with CS
    % assign equal probabilities for each MOEA's operator
    prob1=0.5; prob2=0.5; prob11=0.5;prob12=0.5;
    count11=0; count12=0; count21=0; count22=0;
    % define cycle
%     if n==10
%         CS=50;   
%     else
%         CS=100;
%     end
    CS = 50;
    %% Initalize x
    x= xmin + (xmax-xmin )*rand(PopSize,n);
    %% calculate the fitness function
    fitx = zeros(1, PopSize);
    for i = 1 : PopSize
        fitx(i) = feval(ParamsFunc.FevalName,ParamsFunc.FuncId, x(i,:));
    end
    %fitx = cec14_func(x',I_fno);
    %% update current fitness evaluation
    current_eval =PopSize;
    %% sort
    [fitx, ind]=sort(fitx);
    x=x(ind,:);
    
    %% store the best solution
    bestold=min(fitx);     bestx= x(1,:);
    
    %% fill in each MOEA
    %% DE
    EA_1= x;    EA_obj1= fitx;
    %% ES
    EA_2= x;    EA_obj2= fitx;
    % define CMA-ES parameters
    setting=[];fitness=[];bnd=[];
    [setting,fitness,bnd]= init_cmaes_par(setting,fitness,bnd,EA_obj2,EA_2);
    sa=0;
    %% GA
    EA_3= x;    EA_obj3 = fitx;
    %% initalize the amplifcation factor and crossver rate of each individual

    mu= min(0.95, max(0.1,normrnd(0.5,0.1,1,n1)));
    cr= min(0.95, max(0.1,normrnd(0.5,0.1,1,n1)));

    %% main loop
    if current_eval<max_eval
        stop_con=0;
    end
    while ~stop_con
        
        count_iter=count_iter+1; % to control CS
        if CS ~= 0
        %% check to determine the best MOEA or run all of MOEAs
        if(count_iter==CS+1) % disp(count_iter); disp(current_eval);
            %% fit exponential curve and calculate the expected fitness, of each MOEA, after 2CS genertions
            % hence, decide which MOEA was the best
            indx = fit_exp (success);
            %% print the best MOEA--> this step is optional
%             fprintf(best_EA,'%6.2f\t %12.0f\n', current_eval, indx);
            %% clear the success array
            success=[];
            %% reset probabilities
            prob1=0.5; prob2=0.5; prob11=0.5;prob12=0.5;
            count11=0; count12=0; count21=0; count22=0;
            
        elseif (count_iter==2*CS) && current_eval<=limit_all
           % disp(count_iter); disp(current_eval);
            success=[];   count_iter=1;   add_pop=1;    mubest=2;
            
            [EA_1,EA_2,EA_3,EA_obj1,EA_obj2,EA_obj3] = ...
                inf_sharing(EA_1,EA_2,EA_3,EA_obj1,EA_obj2,EA_obj3, indx,bestx,bestold,f_optimal,mubest,add_pop, ParamsFunc);
            %% sort each MOEA
            [EA_obj1, ind]=sort(EA_obj1);
            EA_1=EA_1(ind,:);
            [EA_obj2, ind]=sort(EA_obj2);
            EA_2=EA_2(ind,:);
            [EA_obj3, ind]=sort(EA_obj3);
            EA_3=EA_3(ind,:);
            
            %% update the xmean to be used for CMA-ES
            setting.xmean = EA_2((1:setting.mu),:)'*setting.weights;
            
            %% reset probabilities to their initial values
            prob1=0.5; prob2=0.5; prob11=0.5;prob12=0.5;
            count11=0; count12=0; count21=0; count22=0;
        end
        end % CS = 0;
        %% MOEA1 --> SAMO-DE
        if (current_eval<max_eval)
            if count_iter<CS || indx==1
                [EA_1, EA_obj1, success,prob11,prob12,count11,count12,bestold,bestx,col,mu,cr] = ...
                    SAMO_DE( EA_1, EA_obj1,success,prob11,prob12,bestold,bestx,col,mu,cr, ParamsFunc);
                
                if count_iter<CS
                    impro1=EA_obj1(1);
                end
            end
        end
        %% MOEA2 --> SAMO-ES
        if (current_eval<max_eval)
            if count_iter<CS ||  indx==2
                count=0;
                [ EA_2, EA_obj2, success, setting,bestold,bestx,col,count,bnd,fitness,sa] = ...
                    SAMO_ES( EA_2, EA_obj2, setting, success, iter,bestold,bestx,col,count,fitness,bnd,sa, ParamsFunc);
                if count_iter<CS
                    impro2=EA_obj2(1);
                end
            end
        end
        
        %% MOEA3 --> SAMO-GA
        if (current_eval<max_eval)
            if count_iter<CS || indx==3
                [ EA_3, EA_obj3, success, prob1,prob2,bestold,bestx,col] = ...
                    SAMO_GA( EA_3, EA_obj3,success, iter, T, prob1,prob2,bestold,bestx,col, ParamsFunc);
                if count_iter<CS
                    impro3=EA_obj3(1);
                end
            end
        end
        
        %% record the success,i.e. the absolute function error of the best solution found and the optimal solution, of each EA
        if count_iter<=CS
            success(count_iter,:) = [abs(impro1-f_optimal)  abs(impro2-f_optimal) abs(impro3-f_optimal)];
        end
        
        iter = iter+1;
        
        if current_eval<max_eval
            stop_con=0;
        else
            stop_con=1;
        end
        
    end
    