function bbUMOEAs
    clear;
    % This program is a full demonstration of a client program
    % for participation in the black box optimization competition.
    % It implements the Nelder Mead algorithm, see
    % http://en.wikipedia.org/wiki/Nelder%E2%80%93Mead_method
    %
    % The algorithm is applied to all problems in a given track;
    % it is demonstrated with the demo account on the trial track.
    % The program shows one possible way to make use of the provided
    % recovery mechanisms. In case of a crash or a power outage the
    % program can be restarted and it will continue the optimization
    % from the saved state. It handles network problems gracefully.

    % configuration
    global LIBNAME; global LIBHFILE;    
    global TRACKNAME;   global USERNAME; global PASSWORD; 
    global LOGIN_DELAY_SECONDS, global LOCK_DELAY_SECONDS;
    LIBNAME = 'bbcomp'; % working directory should include library file(s) and header file
    LIBHFILE = 'bbcomplib.h';   
    LOGFILEPATH = 'logs/';  % path to log files, make sure that it exists and is writable!
    USERNAME = 'chao';      % 'demoaccount';
    PASSWORD = 'chao234';   % 'demopassword';
    TRACKNAME = 'trial';    %'BBComp2015CEC';  %
    LOGIN_DELAY_SECONDS = 10;    %10
    LOCK_DELAY_SECONDS = 60;

    loadBBCompLibrary();

    disp('---------------------------------------------------');
    disp('black box full example competition client in Matlab');
    disp('---------------------------------------------------');
    disp(' ');

    %set configuration options (this is optional)
    result = calllib(LIBNAME,'configure', 1, LOGFILEPATH);
    if (result == 0) 	whenFailed('configure', LIBNAME);   return;	end; 

    safeLogin();

    printTrackNames();   % request the tracks available to this user (this is optional)

    safeSetTrack();

    numProblems = safeGetNumberOfProblems();   % obtain number of problems in the track

    % solve all problems in the track
    %for i=0:(numProblems-1)
    for i = 970
        solveProblem(i);
    end;

    unloadlibrary(LIBNAME);

function solveProblem(problemID)
    global LIBNAME;
    
    % set the problem
    safeSetProblem(problemID);

    % obtain problem properties
    bud = safeGetBudget(problemID);
    dim = safeGetDimension(problemID);
    evals = safeGetEvaluations(problemID);

    % output status
    if (evals == bud)
        disp(['problem ' num2str(problemID) ' already solved']);
        return;
    else
        if (evals == 0)
            disp(['problem ' num2str(problemID) ': starting from scratch']);
        else
            disp(['problem ' num2str(problemID) ': continuing from evaluation ' num2str(evals)]);
      	end;
    end;

    % recover algorithm state from saved evaluations
    % NOTE: if your algorithm uses random numbers, you may want to take care
    % of seed (see http://fr.mathworks.com/help/matlab/ref/rng.html) 
    % to recover the state of the algorithm exactly
    for e=1:evals
        value = libpointer('doublePtr',1e+100);
        x0 = libpointer('doublePtr',zeros(1,dim));
        [result, x0, value] = calllib(LIBNAME,'history',e-1,x0,value);
        if (result == 0)
            disp('WARNING: history failed.\n'); % note: this evaluation is lost
        else
        end;
    end;

    ParamsFunc.FunctionEvaluations = bud - evals;
    ParamsFunc.Dim = dim;
    ParamsFunc.FuncId = problemID;
    ParamsFunc.FevalName = 'safeEvaluate';
    % optimization loop
    tic;
    while (safeGetEvaluations(problemID) < bud)
        %f_xr = safeEvaluate(problemID, xr);
        UMOEAs_Main(ParamsFunc);
    end;
    toc
    
%%% some auxiliary functions %%%    
    
function whenFailed(funcname, LIBNAME)
    disp([funcname '() failed: ' calllib(LIBNAME,'errorMessage')]);    
    unloadlibrary(LIBNAME);

function loadBBCompLibrary()
    global LIBNAME; global LIBHFILE;
    if (libisloaded(LIBNAME) == 1) % first unload if already loaded
        unloadlibrary(LIBNAME);
    end;
    loadlibrary(LIBNAME,LIBHFILE);
    Library_Functions = libfunctions(LIBNAME, '-full') % print library functions
    if (numel(Library_Functions) == 0)
        disp('error loading library');
        return;
    end;

    
function safeLogin()
    global LIBNAME; global USERNAME; global PASSWORD; global LOGIN_DELAY_SECONDS;
    while (1)
        result = calllib(LIBNAME,'login',USERNAME,PASSWORD); 
        if (result ~= 0) return;    end;
        msg = calllib(LIBNAME,'errorMessage');
        disp(['WARNING: login failed: ' msg]);
        pause(LOGIN_DELAY_SECONDS);
        if (strcmp(msg, 'already logged in') == 0) return;  end;
    end

    
function safeSetTrack()
    global TRACKNAME; global LIBNAME; global USERNAME; 
    global PASSWORD; global LOGIN_DELAY_SECONDS;
    while (1)
        result = calllib(LIBNAME,'setTrack',TRACKNAME);
        if (result ~= 0) return;    end;
        disp(['WARNING: setTrack failed: ' calllib(LIBNAME,'errorMessage')]);
        safeLogin();
    end;

    
function printTrackNames()
    global LIBNAME;
    numTracks = calllib(LIBNAME,'numberOfTracks');     
    if (numTracks == 0) whenFailed('numberOfTracks', LIBNAME);    return;	end;
    disp([num2str(numTracks) ' track(s):']);
    for i=1:numTracks
        trackname = calllib(LIBNAME,'trackName',i-1); % indexing from 0
        if (isempty(trackname))         whenFailed('trackName', LIBNAME);       return;
        else                            disp([' ' num2str(i) ': ' trackname]);	end;
    end;

    
function numProblems = safeGetNumberOfProblems()
    global LIBNAME;
    numProblems = calllib(LIBNAME,'numberOfProblems');
    if (numProblems == 0) whenFailed('numberOfProblems', LIBNAME);     return;	end;
    disp(['The track consists of ' num2str(numProblems) ' problems.']);
  
    
function safeSetProblem(problemID)
    global LIBNAME; global LOCK_DELAY_SECONDS;
    while (1)
        result = calllib(LIBNAME,'setProblem',problemID);
        if (result ~= 0) return;    end;
        msg = calllib(LIBNAME,'errorMessage');
        disp(['WARNING: setProblem failed: ' msg]);
        %if (strcmp(msg, 'failed to acquire lock') == 0) 
            pause(LOCK_DELAY_SECONDS);
       % else
            safeSetTrack();
       % end; 
    end;
  
    
function result = safeGetDimension(problemID)
    global LIBNAME;
    while (1)
        result = calllib(LIBNAME,'dimension');
        if (result ~= 0) return;    end;
        disp(['WARNING: dimension failed: ' calllib(LIBNAME,'errorMessage')]);
        safeSetProblem(problemID);
    end;
   
    
function result = safeGetBudget(problemID)
    global LIBNAME;
    while (1)
        result = calllib(LIBNAME,'budget');
        if (result ~= 0) return; end;
        disp(['WARNING: budget failed: ' calllib(LIBNAME,'errorMessage')]);
        safeSetProblem(problemID);
    end;
   
    
function result = safeGetEvaluations(problemID)
    global LIBNAME;
    while (1)
        result = calllib(LIBNAME,'evaluations');
        if (result >= 0) return;    end;
        disp(['WARNING: evaluations failed: ' calllib(LIBNAME,'errorMessage')]);
        safeSetProblem(problemID);
    end;
    
% function value = safeEvaluate(problemID, point)
%     global LIBNAME;
% 	while (1)
%         % query the black box
%         value = libpointer('doublePtr',1e+100);
%         [result, TMP, value] = calllib(LIBNAME,'evaluate',point,value);
% 		if (result ~= 0) return;    end;
% 		disp(['WARNING: evaluate failed: ' calllib(LIBNAME,'errorMessage')]);
% 		safeSetProblem(problemID);
%     end;
    
function value = truncate2bounds(value)
    value = max(0.0, min(1.0, value));
